; #INDEX# =======================================================================================================================
; Nom .........: Translate
; Version .....: 1.0.6
; AutoIt Version : 3.3.6++
; Language ......: Franais
; Description ...: Fonction pour traduire du texte via Google Translate.
; Auteur(s) .....: aurel2108 (aurel2108 at gmail dot com
;
; Merci  PandiPanda pour toute son aide et ses reports de bugs. :)
;
; Version 1.0 : 22/07/2011
;			- Premire version
;
; Version 1.0.1 : 22/07/2011
; 			- Fin de l'utilisation de inet.au3 et _InetGetSource()
;			- Utilisation de InetRead() et BinaryToString()
;
; Version 1.0.2 : 28/07/2011
; 			- Fin de l'utilisation de InetRead() et BinaryToString()
;			- Utilisation du protocole TCP
;			- Rapidit d'excution (16x moins de temps)
;
; Version 1.0.3 : 29/07/2011
;			- Correction du bug d'encodage UTF-8
;			- Correction du bug d'encodage URL
;			- Ajout du retour des diffrentes propositions de traductions (propos par mikell)
;
; Version 1.0.4 : 29/07/2011
;			- Correction d'un bug avec les phrases trop longues.
;
; Version 1.0.5 : 29/07/2011
;			- Amlioration de la fonction UTF8Decode() grce  PandiPanda
;			- Ajout d'un "vrai" exemple de mini-traducteur multi-lingue.
;			- Correction d'un bug avec 'auto' (merci @PandiPanda)
;
; Version 1.0.6 : 30/07/2011
;			- Correction de la fonction URLEncode() grce  PandiPanda
;			- Correction d'un bug avec les sauts de lignes.
; ===============================================================================================================================

#include <Array.au3>
#Include <Timers.au3>

Local Const $debug = false

if $debug = True Then
	#include <GUIConstantsEx.au3>
	#include <WindowsConstants.au3>
EndIf

Global Const $langs[32] = ['fr', 'en', 'de', 'es', 'it', 'pt', 'ca', 'hr', 'da', 'et', 'fi', 'gl', 'cy', 'hu', 'id', 'ga', 'is', 'lt','ms', 'mt', 'nl', 'no', 'pl', 'ro', 'sk', 'sl', 'sv', 'sw', 'tl', 'ts', 'tr', 'vi']

#cs
;Exemple 1 :

$text = 't.'
$timer = _Timer_Init()
$translation = _Translate($text, 'en')
$timer = Round(_Timer_Diff($timer))
MsgBox(64, 'Translate - Exemple 1', 'Temps d''excution : ' & $timer & 'ms' & @CRLF & @CRLF & 'Texte traduit : ' & $translation)

;Exemple 2 :

$text = 'AutoIt v3 is a freeware BASIC-like scripting language designed for automating the Windows GUI and general scripting. It uses a combination of simulated keystrokes, mouse movement and window/control manipulation in order to automate tasks in a way not possible or reliable with other languages (e.g. VBScript and SendKeys).'

$timer = _Timer_Init()
$translation = _Translate($text, 'fr', 'auto', 1)
MsgBox(32, 'Translate - Exemple 2', 'Temps d''excution : ' & Round(_Timer_Diff($timer)) & 'ms', 5)
If IsArray($translation) Then
	_ArrayDisplay($translation, "Translate - Exemple 2")
Else
	MsgBox(32, 'Translate - Exemple 2', $translation)
EndIf
#ce

; #FUNCTION# ====================================================================================================================
; Nom...........: _Translate
; Description ...: Traduire un texte en utilisant Google Translate.
; Syntaxe.........: _Translate($text, $outputLang[, $inputLang, $otherTranslations])
; Paramtres ....: $text - Texte  traduire
;                  $outputLang  - Langue de sortie
;				   $inputLang - [optionnel] Langue d'entre (langue du texte)
;				   $otherTranslations - [optionnel] Retourne l'array de toutes les traductions possibles si la valeur est 1
; Valeurs de retour .: Succs - Retourne le texte traduit
;                 	   chec  - -1 si la langue de sortie n'est pas correcte.
;
;					   Si $otherTranslations est mis  1
;						Succs - Retourne un array contenant la ou les diffrentes traductions
; Auteur ........: aurel2108
; Remarques .......: Si le code source de Google Translate change, la fonction pourrait ne plus fonctionner, si c'est le cas, veuillez le signaler  l'auteur. ;)
;				   $inputLang et $outputLang doivent tre les "initiales" de la langue tel que 'fr' pour le franais, 'en' pour l'anglais, etc...
; Exemple .......: Oui (ci-dessus)
; ===============================================================================================================================

func _Translate($text, $outputLang, $inputLang = 'auto', $otherTranslations = 0)
	if _ArraySearch($langs, $outputLang) <> -1 Then
		if _ArraySearch($langs, $inputLang) == -1 Then
			$inputLang = 'auto'
		EndIf
		$source = _HTTPGet("translate.google.fr", "/?hl=&amp;ie=UTF-8&text=" & UrlEncode($text) & "&sl=" & $inputLang & "&tl=" & $outputLang)
	    if $otherTranslations == 0 Then
			$resultat = StringRegExp($source,'onmouseout="this.style.backgroundColor=(.*?)>(.*?)</span></span></div></div>',2)
			if $resultat == 1 OR $resultat == 2 Then
				$resultat = StringRegExp($source, '<span id=result_box class="short_text">(.*?)</span>', 2)
				if $resultat == 1 OR $resultat == 2 Then
					return -5
				Else
					$resultat = $resultat[1]
				EndIf
			Else
				$resultat = $resultat[2]
			EndIf
			$resultat = StringRegExpReplace($resultat, '<span title="(.*?)"', '')
			$resultat = StringReplace($resultat, '</span>', '')
			$resultat = StringReplace($resultat, ' onmouseover="this.style.backgroundColor=''#ebeff9''" onmouseout="this.style.backgroundColor=''#fff''">', '')
			if $debug == true Then
				GuiCreate("Debug", 800, 600)
				GUISetState()

				$edit = GUICtrlCreateEdit($source, 0, 0, 800, 600)

				While 1
					$msg = GUIGetMsg()
					If $msg = $GUI_EVENT_CLOSE Then ExitLoop
				WEnd
				GUIDelete()
			Else
				if $resultat == 1 Then
					return -2
				Elseif $resultat == 2 Then
					return -3
				Else
					return UTF8Decode($resultat)
				EndIf
			EndIf
		Else
			$autreTrans = StringRegExp($source, '</li></ol></li><li>(.*?)<ol>(.*?)</ol></li></ol></div></div>', 2)
			if $autreTrans == 1 Then
				$resultat = StringRegExp($source,'onmouseout="this.style.backgroundColor=(.*?)>(.*?)</span></span></div></div>',2)
				$resultat = StringRegExpReplace($resultat[2], '<span title="(.*?)"', '')
				$resultat = StringReplace($resultat, '</span>', '')
				$resultat = StringReplace($resultat, ' onmouseover="this.style.backgroundColor=''#ebeff9''" onmouseout="this.style.backgroundColor=''#fff''">', '')
				if $resultat == 1 Then
					return -2
				Elseif $resultat == 2 Then
					return -3
				Else
					return UTF8Decode($resultat)
				EndIf
			ElseIf $autreTrans == 2 Then
				return -4
			Else
				$resultat = StringReplace(UTF8Decode($autreTrans[2]), "</li>", "")
				$resultat = StringRegExpReplace($resultat, '<span title="(.*?)"', '')
				$resultat = StringReplace($resultat, '</span>', '')
				$resultat = StringReplace($resultat, ' onmouseover="this.style.backgroundColor=''#ebeff9''" onmouseout="this.style.backgroundColor=''#fff''">', '')
				$resultat = StringExplode($resultat, '<li>')
				_ArrayDelete($resultat, 1)
				_ArrayDelete($resultat, 0)
				return $resultat
			EndIf
		EndIf
	Else
		return -1
	EndIf
EndFunc

;Internals functions

Func _HTTPGet($sHost, $sPage)
    Local $iSocket = _HTTPConnect($sHost)
    If @error Then Return SetError(1, 0, "")

    Local $sCommand = "GET " & $sPage & " HTTP/1.1" & @CRLF

    $sCommand &= "Host: " & $sHost & @CRLF
    $sCommand &= "User-Agent: Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.1) Gecko/20061010 Firefox/2.0" & @CRLF
    $sCommand &= "Referer: " & $sHost & @CRLF
    $sCommand &= "Connection: close" & @CRLF & @CRLF

    Local $BytesSent = TCPSend($iSocket, $sCommand)
    If $BytesSent = 0 Then Return SetError(2, @error, 0)

    Local $sRecv = "", $sCurrentRecv

    While 1
        $sCurrentRecv = TCPRecv($iSocket, 16)
        If @error <> 0 Then ExitLoop
        If $sCurrentRecv <> "" Then $sRecv &= $sCurrentRecv
    WEnd

    _HTTPShutdown($iSocket)

    Return $sRecv
EndFunc

Func _HTTPConnect($sHost, $iPort=80)
    TCPStartup()

    Local $sName_To_IP = TCPNameToIP($sHost)
    Local $iSocket = TCPConnect($sName_To_IP, $iPort)

    If $iSocket = -1 Then
        TCPCloseSocket($iSocket)
        Return SetError(1, 0, "")
    EndIf

    Return $iSocket
EndFunc

Func _HTTPShutdown($iSocket)
    TCPCloseSocket($iSocket)
    TCPShutdown()
EndFunc

Func StringExplode($sString, $sDelimiter, $sLimit = 0)
    If $sLimit > 0 Then
        ;Replace delimiter with NULL character using given limit
        $sString = StringReplace($sString,$sDelimiter,Chr(0),$sLimit)

        ;Split on NULL character, this will leave the remainder in the last element
        $sDelimiter = Chr(0)
    ElseIf $sLimit < 0 Then
        ;Find delimiter occurence from right-to-left
        $iIndex = StringInStr($sString,$sDelimiter,0,$sLimit)

        If $iIndex Then
            ;Split on left side of string only
            $sString = StringLeft($sString,$iIndex-1)
        EndIf
    EndIf

    Return StringSplit($sString,$sDelimiter,1)
EndFunc

Func UrlEncode($text)

    $text = BinaryToString(StringToBinary($text,4),1)
    $X2 = StringRegExp($text, '&#(.*?);', 3)
    For $i = 0 To UBound($X2) - 1 Step 1
        $text = StringReplace($text, '&#' & $X2[$i] & ';', ChrW($X2[$i]))
    Next
    return StringRegExpReplace($text,'\s','+')
EndFunc

Func UTF8Decode($text)
	$X2 = StringRegExp($text, '&#(.*?);', 3)
	For $i = 0 To UBound($X2) - 1 Step 1
		$text = StringReplace($text, '&#' & $X2[$i] & ';', ChrW($X2[$i]))
	Next
	$text = BinaryToString(StringToBinary($text,1),4)
	return StringReplace($text, "<br>", @CRLF)
EndFunc